import java.util.*;


/**
 * The {@code ServerModel} is the class responsible for tracking the state of
 * the server, including its current users and the channels they are in. This
 * class is used by subclasses of {@link Command} to handle commands from
 * clients, as well as the {@link ServerBackend} to coordinate client connection
 * and disconnection.
 */
public final class ServerModel implements ServerModelApi {
    

    /**
     * Constructs a {@code ServerModel} and initializes any collections
     * needed for modeling the server state.
     */
    public ServerModel() {
        // TODO: Initialize your state here
    }


    //==========================================================================
    // Client connection handlers
    //==========================================================================

    /**
     * Informs the model that a client has connected to the server with the
     * given user ID. The model should update its state so that it can
     * identify this user during later interactions with the model. Any user
     * that is registered with the server (without being later deregistered)
     * should appear in the output of {@link #getRegisteredUsers()}.
     * @param userId the unique ID created by the backend to represent this user
     * @return a {@link Broadcast} informing the user of their new nickname
     */
    public Broadcast registerUser(int userId) {
        String nickname = generateUniqueNickname();
        // TODO: Return broadcast upon user connection
        return null;
    }

    /**
     * Generates a unique nickname of the form "UserX", where X is the
     * smallest non-negative integer that yields a unique nickname for a user.
     * @return the generated nickname
     */
    private String generateUniqueNickname() {
        int suffix = 0;
        String nickname;
        Collection<String> existingUsers = getRegisteredUsers();
        do {
            nickname = "User" + suffix++;
        } while (existingUsers != null && existingUsers.contains(nickname));
        return nickname;
    }

    /**
     * Determines if a given nickname is valid or invalid (contains at least
     * one alphanumeric character, and no non-alphanumeric characters).
     * @param name The channel or nickname string to validate
     * @return true if the string is a valid name
     */
    public static boolean isValidName(String name) {
        if (name == null || name.isEmpty()) {
            return false;
        }
        for (char c : name.toCharArray()) {
            if (!Character.isLetterOrDigit(c)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Informs the model that the client with the given user ID has
     * disconnected from the server. After a user ID is deregistered, the
     * server backend is free to reassign this user ID to an entirely
     * different client. As such, the model should take care to expunge any
     * state pertaining to a user who has been deregistered. Any user that is
     * deregistered (without later being registered) should not appear in the
     * output of {@link #getRegisteredUsers()}. The behavior of this method if
     * the given user ID is not registered with the model is undefined.
     * @param userId the unique ID created by the backend to represent this user
     * @return A {@link Broadcast} informing other clients in the
     * disconnected user's channels that they have disconnected
     */
    public Broadcast deregisterUser(int userId) {
        // TODO: Return broadcast upon user deregistration
        return null;
    }

    

    //==========================================================================
    // Model update functions
    //==========================================================================
    // TODO: Add functions that update your model

    //==========================================================================
    // Server model queries
    // These functions provide helpful ways to test the state of your model.
    // You may also use them in your implementation.
    //==========================================================================

    /**
     * Returns the user ID currently associated with the given nickname.
     * The returned ID is -1 if the nickname is not currently in use.
     * @param nickname The user's nickname
     * @return the id of said user
     */
    public int getUserId(String nickname) {
        // TODO: Return user ID corresponding to nickname
        return -1;
    }

    /**
     * Returns the nickname currently associated with the given user ID.
     * The returned string is null if the user ID is not currently in use.
     * @param userId The ID whose nickname to return.
     * @return The nickname associated with the current ID.
     */
    public String getNickname(int userId) {
        // TODO: Return nickname corresponding to ID
        return null;
    }

    /**
     * Returns a collection of the nicknames of all users that are registered
     * with the server. Provided for testing; modifications to the returned
     * collection should not affect the server state.
     * @return the collection of registered user nicknames
     */
    public Collection<String> getRegisteredUsers() {
        // TODO: Return users connected to server
        return null;
    }

    /**
     * Returns a collection of the names of all the channels that are present
     * on the server. Provided for testing; modifications to the returned
     * collection should not affect the server state.
     * @return the collection of channel names
     */
    public Collection<String> getChannels() {
        // TODO: Return channels on server
        return null;
    }

    /**
     * Returns a collection of the nicknames of all the users in a given
     * channel. The returned collection is empty if no channel with the given
     * name exists. Provided for testing; modifications to the returned
     * collection should not affect the server state.
     * @param channelName The channel whose member nicknames should be returned
     * @return the collection of user nicknames in the current channel
     */
    public Collection<String> getUsers(String channelName) {
        // TODO: Return users in the channel
        return null;
    }

    /**
     * Returns the nickname of the owner of the current channel. The result is
     * {@code null} if no channel with the given name exists. Provided for
     * testing.
     * @param channelName The channel whose owner nickname should be returned
     * @return the nickname of the channel owner
     */
    public String getOwner(String channelName) {
        // TODO: Return owner of the channel
        return null;
    }

}
